@@ -4,9 +4,11 @@ import sys
 from typing import Optional
 
 import click
+import inject
 from click.core import ParameterSource
 
-from db_contrib_tool.evg_aware_bisect import Bisect
+from db_contrib_tool.config import SETUP_REPRO_ENV_CONFIG, SetupReproEnvConfig
+from db_contrib_tool.evg_aware_bisect import Bisect, BisectParameters
 from db_contrib_tool.usage_analytics import CommandWithUsageTracking
 from db_contrib_tool.utils.evergreen_conn import EVERGREEN_CONFIG_LOCATIONS
 
@@ -110,15 +112,24 @@ def bisect(
     ) == ParameterSource.DEFAULT and os.path.isfile(TOOLCHAIN_PYTHON3):
         python_installation = TOOLCHAIN_PYTHON3
 
-    bisect_cmd = Bisect(
-        branch=branch,
-        lookback=lookback,
-        evergreen_config=evergreen_config,
-        variant=variant,
-        script=script,
-        debug=debug,
-        python_installation=python_installation,
-    )
+    def dependencies(binder: inject.Binder) -> None:
+        """Dependencies for bisect command execution."""
+        binder.bind(SetupReproEnvConfig, SETUP_REPRO_ENV_CONFIG)
+        binder.bind(
+            BisectParameters,
+            BisectParameters(
+                branch=branch,
+                lookback=lookback,
+                variant=variant,
+                script=script,
+                python_installation=python_installation,
+                debug=debug,
+                evergreen_config=evergreen_config,
+            ),
+        )
+
+    inject.configure(dependencies)
+    bisect_cmd = inject.instance(Bisect)
 
     success = bisect_cmd.execute()
     if not success:
