@@ -7,7 +7,7 @@ from typing import Dict, List, NamedTuple, Optional
 
 import inject
 import structlog
-from evergreen import Version
+from evergreen import Task, Version
 from requests.exceptions import HTTPError
 
 from db_contrib_tool.config import SETUP_REPRO_ENV_CONFIG_FILE, DownloadTarget, SetupReproEnvConfig
@@ -88,6 +88,20 @@ class RequestTarget(NamedTuple):
         return f"{self.request_type}({self.identifier})"
 
 
+class TaskDependencyParser:
+    """Parse dependencies for a given task."""
+
+    @staticmethod
+    def get_compile_dependency(task: Task, binary_task_name: str) -> Optional[str]:
+        """Get the ID of the task to download the binaries."""
+        if task.depends_on is not None:
+            for dependency in task.depends_on:
+                task_id = dependency["id"]
+                if binary_task_name in task_id:
+                    return task_id
+        return None
+
+
 class ArtifactDiscoveryService:
     """A service for finding artifacts."""
 
@@ -132,7 +146,6 @@ class ArtifactDiscoveryService:
         :param starting_commit: When scanning multiple commits, start scanning after this commit.
         :return: Links to found artifacts.
         """
-        artifact_urls = None
         if request.request_type == RequestType.EVG_VERSION:
             artifact_urls = self.find_artifact_urls_for_version(
                 request.identifier, requested_variant, target, ignore_failed_push
@@ -140,8 +153,19 @@ class ArtifactDiscoveryService:
 
         elif request.request_type == RequestType.EVG_TASK:
             task = self.evg_service.get_task(request.identifier)
+
+            # For tasks that depend on other tasks to generate artifacts, we go one step
+            # further to ensure we get the correct variant that has the artifacts.
+            compile_task_id = TaskDependencyParser.get_compile_dependency(
+                task, self.setup_repro_config.get_default_binary_task()
+            )
+
+            if compile_task_id is not None:
+                task = self.evg_service.get_task(compile_task_id)
+
             if requested_variant is None:
                 requested_variant = task.build_variant
+
             artifact_urls = self.find_artifact_urls_for_version(
                 task.version_id, requested_variant, target, ignore_failed_push
             )
