@@ -1,7 +1,7 @@
 """Setup MongoDB repro environment."""
 import os
 import re
-from typing import List, NamedTuple, Optional
+from typing import List, NamedTuple, Optional, Set
 
 import inject
 import structlog
@@ -9,10 +9,16 @@ import structlog
 from db_contrib_tool.clients.download_client import DownloadError
 from db_contrib_tool.clients.file_service import FileService
 from db_contrib_tool.clients.resmoke_proxy import ResmokeProxy
-from db_contrib_tool.config import WINDOWS_BIN_PATHS_FILE, DownloadTarget
+from db_contrib_tool.config import (
+    CONTINUOUS_RELEASE_ALIAS,
+    LTS_RELEASE_ALIAS,
+    WINDOWS_BIN_PATHS_FILE,
+    DownloadTarget,
+)
 from db_contrib_tool.services.evergreen_service import EvergreenService
 from db_contrib_tool.setup_repro_env.artifact_discovery_service import (
     ArtifactDiscoveryService,
+    EvgURLInfo,
     RequestTarget,
     RequestType,
 )
@@ -48,8 +54,6 @@ class SetupReproParameters(NamedTuple):
     * variant: Build Variant to download from.
 
     * versions: List of items to download.
-    * install_last_lts: If True download last LTS version of mongo.
-    * install_last_continuous: If True download last continuous version of mongo.
     * ignore_failed_push: Download version even if the push task failed.
     * fallback_to_master: Should the latest master be downloaded if the version doesn't exist.
 
@@ -64,8 +68,6 @@ class SetupReproParameters(NamedTuple):
     variant: Optional[str]
 
     versions: List[str]
-    install_last_lts: bool
-    install_last_continuous: bool
     ignore_failed_push: bool
     fallback_to_master: bool
 
@@ -86,6 +88,22 @@ class SetupReproParameters(NamedTuple):
         )
 
 
+class DownloadRequest(NamedTuple):
+    """Class representing the request to download a repro environment."""
+
+    bin_suffix: str
+    discovery_request: RequestTarget
+    urls_info: EvgURLInfo
+
+    def __eq__(self, other: object) -> bool:
+        """Download objects are identical if the URLs and name suffix are identical."""
+        if isinstance(other, self.__class__):
+            return (other.urls_info.urls == self.urls_info.urls) and (
+                other.bin_suffix == self.bin_suffix
+            )
+        return False
+
+
 class SetupReproOrchestrator:
     """Orchestrator for setting up repro environments."""
 
@@ -113,13 +131,16 @@ class SetupReproOrchestrator:
         self.artifact_discovery_service = artifact_discovery_service
         self.file_service = file_service
 
-    def interpret_request(self, request: str) -> RequestTarget:
+    def interpret_discovery_request(self, request: str) -> RequestTarget:
         """
         Translate the request from the user into an item we can understand.
 
         :param request: Request from user.
         :return: Targeted request to download.
         """
+        if request in [LTS_RELEASE_ALIAS, CONTINUOUS_RELEASE_ALIAS]:
+            return self._get_release_versions(request)
+
         if request in KNOWN_BRANCHES or BRANCH_RE.match(request):
             return RequestTarget(RequestType.GIT_BRANCH, request)
 
@@ -137,46 +158,31 @@ class SetupReproOrchestrator:
 
         return RequestTarget(RequestType.GIT_COMMIT, request)
 
-    def interpret_requests(
-        self, request_list: List[str], last_lts: bool, last_continuous: bool
-    ) -> List[RequestTarget]:
+    def interpret_discovery_requests(self, request_list: List[str]) -> List[RequestTarget]:
         """
-        Translate all the requests from the user into items we can understand.
+        Translate all the requests from the user into a request for discovering the download information.
 
         :param request_list: Requests from user.
-        :param last_lts: Should 'last lts' version be included.
-        :param last_continuous: Should the 'last continuous' version be included.
         :return: List of targeted request to download.
         """
-        requests = [self.interpret_request(request) for request in request_list]
-        if last_lts or last_continuous:
-            requests.extend(self._get_release_versions(last_lts, last_continuous))
+        requests = [self.interpret_discovery_request(request) for request in request_list]
 
         return requests
 
-    def _get_release_versions(
-        self, install_last_lts: Optional[bool], install_last_continuous: Optional[bool]
-    ) -> List[RequestTarget]:
+    def _get_release_versions(self, version_alias: str) -> RequestTarget:
         """
-        Create a list of multiversion versions that should be included.
+        Create a multiversion version that should be included.
 
-        :param install_last_lts: True if the last LTS version should be included.
-        :param install_last_continuous: True if the last continuous version should be included.
-        :return: List of which multiversion versions should be included.
+        :return: multiversion version to include.
         """
         multiversionconstants = self.resmoke_proxy.get_multiversion_constants()
+
         releases = {
-            multiversionconstants.last_lts_fcv: install_last_lts,
-            multiversionconstants.last_continuous_fcv: install_last_continuous,
+            LTS_RELEASE_ALIAS: multiversionconstants.last_lts_fcv,
+            CONTINUOUS_RELEASE_ALIAS: multiversionconstants.last_continuous_fcv,
         }
         LOGGER.debug("LTS and continuous release inclusions", releases=releases)
-        out = {
-            RequestTarget.previous_release(version)
-            for version, requested in releases.items()
-            if requested
-        }
-
-        return list(out)
+        return RequestTarget.previous_release(releases[version_alias])
 
     @staticmethod
     def _get_bin_suffix(version: str, evg_project_id: str) -> str:
@@ -206,11 +212,7 @@ class SetupReproOrchestrator:
         :param setup_repro_params: Setup repro env parameters.
         :return: Whether succeeded or not.
         """
-        request_list = self.interpret_requests(
-            setup_repro_params.versions,
-            setup_repro_params.install_last_lts,
-            setup_repro_params.install_last_continuous,
-        )
+        discovery_request_list = self.interpret_discovery_requests(setup_repro_params.versions)
 
         downloaded_versions = []
         failed_requests = []
@@ -219,13 +221,15 @@ class SetupReproOrchestrator:
         download_target = setup_repro_params.get_download_target()
         LOGGER.info("Search criteria", search_criteria=download_target)
 
-        for request in request_list:
-            LOGGER.info("Setting up request", request=request)
+        download_request_set: Set[DownloadRequest] = set()
+
+        for discovery_request in discovery_request_list:
+            LOGGER.info("Setting up request", request=discovery_request)
             LOGGER.info("Fetching download URLs from Evergreen")
 
             try:
                 urls_info = self.artifact_discovery_service.find_artifacts(
-                    request,
+                    discovery_request,
                     setup_repro_params.variant,
                     download_target,
                     setup_repro_params.ignore_failed_push,
@@ -233,27 +237,46 @@ class SetupReproOrchestrator:
                 )
 
                 if urls_info is None:
-                    failed_requests.append(request)
-                    LOGGER.warning("Unable to find artifacts for request", request=request)
+                    failed_requests.append(discovery_request)
+                    LOGGER.warning(
+                        "Unable to find artifacts for request", request=discovery_request
+                    )
                     continue
 
-                bin_suffix = self._get_bin_suffix(request.identifier, urls_info.project_identifier)
+                bin_suffix = self._get_bin_suffix(
+                    discovery_request.identifier, urls_info.project_identifier
+                )
+                download_request = DownloadRequest(bin_suffix, discovery_request, urls_info)
+                download_request_set.add(download_request)
+                print(download_request)
+            except evergreen_conn.EvergreenConnError:
+                failed_requests.append(discovery_request)
+                LOGGER.error(
+                    "Failed to discover items to download", request=discovery_request, exc_info=True
+                )
+
+        for download_request in download_request_set:
+            try:
                 linked_dir = self.artifact_download_service.download_and_extract(
-                    urls_info.urls,
-                    bin_suffix,
-                    urls_info.evg_version_id,
+                    download_request.urls_info.urls,
+                    download_request.bin_suffix,
+                    download_request.urls_info.evg_version_id,
                     setup_repro_params.download_options,
                 )
                 if linked_dir:
                     link_directories.append(linked_dir)
-                downloaded_versions.append(urls_info.evg_version_id)
-                LOGGER.info("Setup request completed", request=request)
+                downloaded_versions.append(download_request.urls_info.evg_version_id)
+                LOGGER.info("Setup request completed", request=download_request)
             except (
                 evergreen_conn.EvergreenConnError,
                 DownloadError,
             ):
-                failed_requests.append(request)
-                LOGGER.error("Setup request failed", request=request, exc_info=True)
+                failed_requests.append(download_request.discovery_request)
+                LOGGER.error(
+                    "Setup request failed",
+                    request=download_request.discovery_request,
+                    exc_info=True,
+                )
 
         if is_windows():
             self.file_service.write_windows_install_paths(WINDOWS_BIN_PATHS_FILE, link_directories)
@@ -267,8 +290,9 @@ class SetupReproOrchestrator:
                 target_file=os.path.abspath(setup_repro_params.evg_version_file),
             )
 
-        if len(downloaded_versions) < len(request_list):
+        if failed_requests:
             LOGGER.error("Some requests were not able to setup.", failed_requests=failed_requests)
             return False
-        LOGGER.info("Downloaded versions", request_list=request_list)
+
+        LOGGER.info("Downloaded versions", request_list=discovery_request_list)
         return True
